"""Client for API V2.

See https://github.com/actigraph/CentrePoint3APIDocumentation.
"""
import logging
from typing import Any, Dict, List, Union

import requests

from actiapi import ActiGraphClient


class ActiGraphClientV3(ActiGraphClient):
    """Client for CentrePoint V3 API."""

    BASE_URL = "https://api.actigraphcorp.com"
    AUTH_API = "https://auth.actigraphcorp.com/connect/token"

    @staticmethod
    def _generate_headers(token: str, raw: bool = False):
        headers = {}
        if not raw:
            headers["Accept"] = "application/json"
            headers["Content-Type"] = "application/json"
        headers["Authorization"] = f"Bearer {token}"
        return headers

    def _get_access_token(self, scope: str):
        endpoint = self.AUTH_API
        request_body = {
            "client_id": self.api_access_key,
            "client_secret": self.api_secret_key,
            "scope": scope,
            "grant_type": "client_credentials",
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}

        response = requests.post(
            endpoint, data=request_body, headers=headers, verify=True
        )
        try:
            return response.json()["access_token"]
        except KeyError:
            raise RuntimeError(
                "No access token! Make sure you have API_ACCESS_KEY and API_SECRET_KEY!"
            )

    def get_files(
        self,
        user: Union[int, str],
        study_id: int,
    ) -> List[str]:
        """Return download URLs to raw AVRO files.

        Parameters
        ----------
        user:
            User id
        study_id:
            Id of the study
        """
        token = self._get_access_token(
            "DataAccess",
        )
        results = self._get_paginated(
            f"/dataaccess/v3/files/studies/{study_id}/subjects/{user}"
            f"/raw-accelerometer?fileFormat=avro&",
            token,
        )

        results = [x["downloadUrl"] for x in results]

        return results

    def get_study_metadata(self, study_id) -> List[Dict[str, Any]]:
        """Save all study metadata to file.

        Parameters
        ----------
        study_id:
            Id of the study
        """
        token = self._get_access_token("CentrePoint")

        results = self._get_paginated(
            f"/centrepoint/v3/Studies/{study_id}/Subjects?", token
        )
        return results

    def get_minute_summary(
        self, user: Union[int, str], study_id: int
    ) -> List[Dict[str, Any]]:
        """Return minute level data.

        Parameters
        ----------
        user:
            User id
        study_id:
            Id of the study
        """
        token = self._get_access_token(
            "Analytics",
        )
        results = self._get_paginated(
            f"/analytics/v3/Studies/{study_id}/Subjects/{user}/MinuteSummaries?", token
        )
        return results

    def get_daily_summary(
        self, user: Union[int, str], study_id: int
    ) -> List[Dict[str, Any]]:
        """Return daily summary data.

        Parameters
        ----------
        user:
            User id
        study_id:
            Id of the study
        """
        token = self._get_access_token(
            "Analytics",
        )
        results = self._get_paginated(
            f"/analytics/v3/Studies/{study_id}/Subjects/{user}/DailyStatistics?", token
        )
        return results

    def _get_paginated(self, request: str, token: str):
        results = []
        offset = 0
        limit = 100
        while True:
            paginated_request = f"{request}offset={offset}&limit={limit}"
            headers = self._generate_headers(token)
            response = requests.get(
                self.BASE_URL + paginated_request,
                headers=headers,
            )
            reply = response.json()
            total_count = reply["totalCount"]

            for item in reply["items"]:
                results.append(item)
            if offset + limit >= total_count:
                break
            offset += limit
        if len(results) == 0:
            logging.error("No raw data found.")
            return []
        return results



"""A library for interacting with ActiGraph's API."""


class ActiGraphClient:
    """Base classe for ActiGraph API clients.

    Parameters
    ----------
    api_access_key
        API access key
    api_secret_key
        API secret key
    """

    def __init__(self, api_access_key: str, api_secret_key: str):
        """Initialize client."""
        self.api_access_key = api_access_key
        self.api_secret_key = api_secret_key



"""Client for API V2.

See https://github.com/actigraph/StudyAdminAPIDocumentation.
"""

import base64
import hashlib
import hmac
from datetime import datetime
from email.utils import formatdate
from typing import Any, Dict, List

import requests

from actiapi import ActiGraphClient


class ActiGraphClientV2(ActiGraphClient):
    """Client for CentrePoint V2 API."""

    BASE_URL = "https://studyadmin-api.actigraphcorp.com"

    def _HMACSHA256Base64(self, message: str):
        message_bytes = bytes(message, "utf-8")
        secret = bytes(self.api_secret_key, "utf-8")
        signature = base64.b64encode(
            hmac.new(secret, message_bytes, digestmod=hashlib.sha256).digest()
        )
        return signature.decode()

    def _generate_headers(
        self,
        http_verb,
        resource_uri,
    ):
        """Formulate http headers."""
        cur_date = formatdate(timeval=None, localtime=False, usegmt=True)
        cur_date_iso = datetime.utcnow().isoformat().split(".")[0] + "Z"
        string_to_sign = "\n".join(
            [
                http_verb,
                "",
                "",
                cur_date_iso,
                "/".join([ActiGraphClientV2.BASE_URL, resource_uri]),
            ]
        )

        signed_string = self._HMACSHA256Base64(string_to_sign)

        headers = {
            "Authorization": "AGS " + self.api_access_key + ":" + signed_string,
            "Date": cur_date,
        }
        return headers

    def get_study_metadata(self, study_id: str) -> List[Dict[str, Any]]:
        """Download list of subjects and their metadata."""
        resource_uri_subject_metadata = f"v1/studies/{study_id}/subjects"
        headers_subject_metadata = self._generate_headers(
            "GET", resource_uri_subject_metadata
        )
        subject_metadata_str = requests.get(
            "/".join([ActiGraphClientV2.BASE_URL, resource_uri_subject_metadata]),
            headers=headers_subject_metadata,
        )
        subject_metadata = subject_metadata_str.json()
        return subject_metadata

    def get_files(self, subject_id) -> List[Dict[str, Any]]:
        """Get file list for given subject."""
        resource_uri_subject_rawdata = "v1/subjects/" + str(subject_id) + "/dataFiles"
        headers_subject_rawdata = self._generate_headers(
            "GET", resource_uri_subject_rawdata
        )
        subject_rawdata = requests.get(
            "/".join([ActiGraphClientV2.BASE_URL, resource_uri_subject_rawdata]),
            headers=headers_subject_rawdata,
        ).json()

        return subject_rawdata["RAW"]

    def get_url(self, data_file_id) -> str:
        """Get the URL for a file based on its id."""
        resource_uri_rawdata_url = f"v1/datafiles/{data_file_id}/DownloadUrl"
        headers_rawdata_url = self._generate_headers("GET", resource_uri_rawdata_url)
        rawdata_url = requests.get(
            "/".join([ActiGraphClientV2.BASE_URL, resource_uri_rawdata_url]),
            headers=headers_rawdata_url,
        )

        rawdata_dict = rawdata_url.json()
        return rawdata_dict["DownloadURL"]
